#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include "vector.h"

vector vect_new(int size)
{
  vector v;

  if (size <= 0)
    {
      v.size_ = -1;
      v.data_ = NULL;
      printf("error : size <= 0\n");
      return v;
    }

  v.size_ = size;
  v.data_ = (double *)malloc(size * sizeof(double));
  if (!v.data_)
    {
      v.size_ = -1;
      printf("error : allocation failed\n");
      return v;
    }

  return v;
}

void vect_del(vector *v)
{
  if (v->data_ != NULL)
    free(v->data_);
}

void vect_set(vector *v, int idx, double val)
{
  if ((idx < 0) || (idx >= v->size_))
    {
      printf("error : wrong index\n");
      return;
    }

  v->data_[idx] = val;
}

double vect_get(vector *v, int idx)
{
  if ((idx < 0) || (idx >= v->size_))
    {
      printf("error : wrong index\n");
      return 0.0;
    }

  return v->data_[idx];
}

void vect_disp(vector *v)
{
  int i;

  printf("(");

  for (i = 0; i < v->size_; i++)
    {
      printf("%f", v->data_[i]);
      if (i != (v->size_ - 1))
	printf(", ");
    }

  printf(")\n");
}

vector vect_add(vector *v1, vector *v2)
{
  vector v;
  int i;

  if (v1->size_ != v2->size_)
    {
      v.size_ = -1;
      v.data_ = NULL;
      printf("error : addition of vectors of different size\n");
      return v;
    }

  v = vect_new(v1->size_);
  for (i = 0; i < v.size_; i++)
    {
      v.data_[i] = v1->data_[i] + v2->data_[i];
    }

  return v;
}

vector vect_sub(vector *v1, vector *v2)
{
  vector v;
  int i;

  if (v1->size_ != v2->size_)
    {
      v.size_ = -1;
      v.data_ = NULL;
      printf("error : substraction of vectors of different size\n");
      return v;
    }

  v = vect_new(v1->size_);
  for (i = 0; i < v.size_; i++)
    {
      v.data_[i] = v1->data_[i] - v2->data_[i];
    }

  return v;
}

double vect_inner_product(vector *v1, vector *v2)
{
  double ip;
  int i;

  if (v1->size_ != v2->size_)
    {
      ip = 0.0;
      printf("error : inner product of vectors of different size\n");
      return ip;
    }

  ip = 0.0;
  for (i = 0; i < v1->size_; i++)
    {
      ip += v1->data_[i] * v2->data_[i];
    }

  return ip;
}

double vect_norm2(vector *v)
{
  return sqrt(vect_inner_product(v, v));
}
